--[[ 

HOW THE MOD JOINS A CHANNEL

  1. Loads the value of the variable "AnathemaNef_Channel" from disk, overwriting the default value of 
     this file.

  2. When the user executes a console command
       - command handler executes Comms.join() before handling the command
         - Comms.join() sets the channel to the argument if any 
           (not the case when calling from command handler)
         - if Comms.channelName is nil, defaults to the value of AnathemaNef_Channel,
           which is value loaded from disk, or default value "AnathemaNef".
         - Attemps to join channel if we are not already on it.
           If it fails, set AnathemaNef_Channel to the default "AnathemaNef".
--]]

-- default channel
AnathemaNef_Channel = "AnathemaNef"
AnathemaNef_DefaultChannel = "AnathemaNef"

-- Version info
AnathemaNef_Version = "1.3"

-- killed mobs
AnathemaNef_Mobs = { }

-- recorded mobs
AnathemaNef_RecordedMobs = { }


-- LOGGER ==========================================================================================
Logger = {
  
  traceLevel = 0,
  debugLevel = 1,
  infoLevel  = 2,
  warnLevel  = 3,  
  
  prefix = { 
    [0] = "  [trace] ",
    [1] = "  [debug] ",
    [2] = "  [info] ",
    [3] = "  [warn] "
  },

  trace = function(msg) Logger.output(msg, Logger.traceLevel) end,
  debug = function(msg) Logger.output(msg, Logger.debugLevel) end,
  info  = function(msg) Logger.output(msg, Logger.infoLevel)  end,
  warn  = function(msg) Logger.output(msg, Logger.warnLevel)  end,
  
  output = function(msg, lvl)
    -- do nothing if not on this logger level
    if (lvl <= Status.debugLevel) then 
      return 
    end
    -- get nil and boolean as string
    if (msg ==nil) then 
      msg="nil"
    elseif (type(msg)=="boolean") then 
      msg = Status.getAsString(msg) 
    end
    -- console output
    Comms.console(Logger.prefix[lvl]..msg)
  end
  
}


-- MOBDATA =========================================================================================
-- Attemps to reduce bandwith using pairs name/id to replace full names.
-- *NOT* used to store killed mobs.
MobData = {

  getNameById = function(id)
    for k, v in Counter.pairsByKeys(MobCodes) do
      -- Todo: use alphabetic comparison to break the loop
      -- requires assigning automatic indexes after sorting the table
      if (""..v == ""..id) then
        return k
      end
    end
    -- return id if not found
    return id
  end,
  
  getIdByName = function(name)
    for k, v in Counter.pairsByKeys(MobCodes) do
      -- Todo: use alphabetic comparison to break the loop
      if (""..k == ""..name) then
        Logger.debug("key found: "..k)
        return v
      end
    end
    -- return name if not found
    return name
  end
  
}


-- COUNTER =========================================================================================
-- Mob counter.
Counter = {
  size = 0,
  discarded = 0,
  DISCARDED_INTERVAL = 300,
  
  lastMob = { millis=0, name="", reporterName=nil},


  -- Decides if this mobName is a duplicate report of last death.
  -- Note that this may lead to wrong discards. eg: weak mobs with same name dieing in less
  -- than DISCARDED_INTERVAL milliseconds, due to an aoe.
  isDuplicate = function(mobName, reporterName)
    local duplicate = true

    -- not first report of this name?
    if (AnathemaNef_Mobs[mobName] ~= nil) then
      -- if the reporter is the same it is not duplicate
      if (AnathemaNef_Mobs[mobName]["reporter"] == reporterName) then
        duplicate = false
      else
        -- if this reporter is not the same, accept only if interval > 300ms
        local interval = debugprofilestop() - AnathemaNef_Mobs[mobName]["last"]
        Logger.debug("interval from last report of this mob: "..interval)
        if (interval > Counter.DISCARDED_INTERVAL) then
          duplicate = false
        end
      end
    else
      -- first report of this mob
      Logger.trace("first report of this mob")
      duplicate = false
    end

    if (duplicate) then
      Counter.discarded = Counter.discarded + 1
      Logger.trace("discarded")
    else
      Logger.trace("NOT discarded")
    end

    return duplicate
  end,


  -- Attemp to add the kill of <mobName>.
  -- @return: true if the mob was added (it wasnt a duplicate), false otherwise.
  add = function (mobName, reporterName)
    local b = Counter.isDuplicate(mobName, reporterName)
    if (not b) then
      Counter.reallyAdd(mobName, reporterName)
      return true
    else
      return false
    end
  end,
  
  
  -- Adds a mob to the counter.
  reallyAdd = function(mobName, reporterName)
      if (AnathemaNef_Mobs[mobName] == nil) then
        AnathemaNef_Mobs[mobName] = { alias, total, reporter, last }
        AnathemaNef_Mobs[mobName]["total"] = 1
      else
        AnathemaNef_Mobs[mobName]["total"] = AnathemaNef_Mobs[mobName]["total"] + 1
      end
      
      -- update last reporter and timestamp
      AnathemaNef_Mobs[mobName]["reporter"]=reporterName
      AnathemaNef_Mobs[mobName]["last"]=debugprofilestop()
      
      -- increase global counter
      Counter.size = Counter.size + 1
      Logger.debug("")
  end,

  
  -- return number of deaths
  getSize = function()
    return Counter.size
  end,

  
  -- reset counter
  reset = function()
    AnathemaNef_Mobs = {}
    Counter.size = 0
    Counter.discarded = 0
    debugprofilestart()
  end,
  
  record = function()
    -- save an ordered copy
    local i = 0
    for name, line in Counter.pairsByKeys(AnathemaNef_Mobs) do
      AnathemaNef_RecordedMobs[name]=i
      i = i + 1
    end
    -- get as string
    --local s = ""
    --for name, index in Counter.pairsByKeys(AnathemaNef_RecordedMobs) do
    --  s = s..name..": "..index.."\n"
    --end
    --return s
  end,

  -- Returns a string with the info of this counter (number and kind of mobs killed).
  toString = function ()
    local s = "** Kills in this session ***\n\n"
    for name, kills in Counter.pairsByKeys(AnathemaNef_Mobs) do
      s = s..name..": "..kills["total"].."\n"
    end
    s = s.."Total: "..Counter.size
    Logger.trace(s)
    return s
  end,
  
  -- Sort the table of mobs in alphabetical order.
  -- Taken from "Programming in Lua: Sort" at http://www.lua.org/pil/19.3.html
  pairsByKeys = function (t, f)
    local a = {}
    for n in pairs(t) do table.insert(a, n) end
    table.sort(a, f)
    local i = 0
    local iter = function()
      i = i + 1
      if a[i] == nil then return nil
      else return a[i], t[a[i]]
      end
    end
    return iter
  end

}


-- STATUS ==========================================================================================
-- Status and command handler functions.
Status = {
  
  -- registered with events?
  registered = false,

  -- report/announce deaths?
  reportFlag = false,
  announceFlag = false,

  -- output debug info?
  debugLevel = 4,
  debugLevelDiz = {
    [0] = "trace",
    [1] = "debug",
    [2] = "info",
    [3] = "warn",
    [4] = "off"
  },
  
  -- just gather data, dont show anything
  silentFlag = false,

  -- mod usage
  usage = [[ 
/atmnef <command>, where command can be
  announce - announce kills (sent by reporters)
  report   - report kills (to be digested by the announcer)
  
  detail - show kind and numbers of death mobs
  reset  - reset the counter
  off    - turn off the mod
  silent - reports on console, not in party/raid channel
  check  - show a list of who is in channel and what their role is
  debug  - pest the console with weird debug info
  ]],
    
  -- toggle debug flag
  debug = function()
    Status.debugLevel = Status.debugLevel + 1
    if (Status.debugLevel==5) then 
      Status.debugLevel = 0
    end
  end,
  
  -- toggle announce flag
  announce = function()
    Status.announceFlag = not Status.announceFlag
    Status.updateHooks()
  end,
  
  -- toggle silent flag
  silent = function()
    Status.silentFlag = not Status.silentFlag
  end,
  
  -- toggle report flag
  report = function()
    Status.reportFlag = not Status.reportFlag
    Status.updateHooks()
  end,
  
  -- turn off the mod
  off = function()
    Status.reportFlag = false
    Status.announceFlag = false
    Status.debugLevel = 0
    Status.updateHooks()
  end,
  
  -- get string with status of all flags
  getStatus = function()
    s = "AnathemaNef: "
    s = s.."announce="..Status.getAsString(Status.announceFlag)
    s = s..", report="..Status.getAsString(Status.reportFlag)
    s = s..", silent="..Status.getAsString(Status.silentFlag)
    s = s..", debug="..Status.debugLevelDiz[Status.debugLevel]
    s = s..", channel="..Comms.channelName
    return s
  end,
  
  -- register as listener for events
  register = function()
    if (not registered) then
      AnathemaNef:RegisterEvent("VARIABLES_LOADED")
      AnathemaNef:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH")
      -- AnathemaNef:RegisterEvent("CHAT_MSG_CHANNEL")
      AnathemaNef:RegisterEvent("CHAT_MSG_MONSTER_YELL")
      AnathemaNef:RegisterEvent("CHAT_MSG_ADDON")
	    -- AnathemaNef:RegisterEvent("CHAT_MSG_RAID");
	    -- AnathemaNef:RegisterEvent("CHAT_MSG_PARTY");
      registered = true
    end
  end,
  
  -- unregister as listener for events
  unregister = function()
    if (registered) then 
      AnathemaNef:UnregisterEvent("VARIABLES_LOADED")
      AnathemaNef:UnregisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH")
      -- AnathemaNef:UnregisterEvent("CHAT_MSG_CHANNEL")
      AnathemaNef:UnregisterEvent("CHAT_MSG_MONSTER_YELL")
      AnathemaNef:UnregisterEvent("CHAT_MSG_ADDON")
	    -- AnathemaNef:UnregisterEvent("CHAT_MSG_RAID");
	    -- AnathemaNef:UnregisterEvent("CHAT_MSG_PARTY");
      registered = false
    end
  end,
  
  -- call register/unregister when appropiate
  updateHooks = function()
    if ((Status.reportFlag) or (Status.announceFlag)) then
      Status.register()
    else
      Status.unregister()
    end
  end,
  
  -- get boolean as string
  getAsString = function(b)
    if(b) then
      return "on"
    else
      return "off"
    end
  end
}


-- COMMUNICATION ===================================================================================
-- Communication. 
Comms = {
  ANNOUNCE_DEATH_PREFIX="ATMNEF ",
  BROADCAST_PREFIX="AnathemaNef: Setting this group channel to ",
  CHECK_PREFIX="AnathemaNef: Status check requested by ",
  REPLY_PREFIX="AnathemaNef: Reply ",
  CHECK_REPORT_PREFIX="AnathemaNef: Check result - ",
  channelName = nil,
    
  
  -- Set and join channel if not already on it.
  -- @param: channel Channel to set and join. Optional parameter.
  join = function(c)
  
    -- set channel if called with an argument
    if (c) then
      -- first, leave current channel if any
      if (Comms.channelName) then
        local id, name = GetChannelName(Comms.channelName)
        if (id ~= 0) then
          LeaveChannelByName(Comms.channelName)
          Logger.debug("leaving "..Comms.channelName)
        end
      end
      Logger.debug("setting channel to "..c)
      Comms.channelName = c
    end
    
    -- set a channel name if there isnt any
    if (not Comms.channelName) then
      -- @TODO: get default channel from CT *if* we are on raid and CT is installed
      -- check that CT doesnt return a nil channel
      --local id, name = GetChannelName(CT_RA_Channel);
      Comms.channelName = AnathemaNef_Channel
      Logger.debug("Setting default channel to "..Comms.channelName)
    end
    
    -- attemp to join if we are not already on it
    if (0 == GetChannelName(Comms.channelName)) then
      -- join Comms.channelName
      Comms.console("AnathemaNef: joining "..Comms.channelName)
      JoinChannelByName(Comms.channelName) --, nil, DEFAULT_CHAT_FRAME:GetID());
      
      -- check we really joined the channel
      local id, name = GetChannelName(Comms.channelName)
      if (id == 0) then
        Logger.debug("join failed! channel was "..Comms.channelName)
        Comms.console("AnathemaNef: Cannot join channel "..Comms.channelName.."!")
        Comms.console("AnathemaNef: Chek that you're using a valid name and that you are not already in 10 channels.")
        Comms.console("AnathemaNef: Some helpful commands: /chatlist, /join channelname, /leave channelname")
        AnathemaNef_Channel = AnathemaNef_DefaultChannel
      else
        Comms.console("AnathemaNef: Successfully joined channel "..Comms.channelName.."!")
        Logger.debug("join successful! channel is "..id.."."..Comms.channelName)
      end
    end

  end,
  
  
-- DISPLAY MSG ============================================
 
  -- send msg through mod channel
  send = function(prefix, msg)
    -- local id, name = GetChannelName(Comms.channelName)
    -- SendChatMessage(msg, "CHANNEL", nil, id);
    
    SendAddonMessage(ANNOUNCE_DEATH_PREFIX, msg, "RAID");
  end,
  
  -- Displays this msg on a visible channel.
  -- Channel preference is raid, party, console.
  -- New line codes are replaced by repeated calls to SendChatMessage.
  display = function(msg)
    if (GetNumRaidMembers()>0) then
      for word in string.gfind(msg, "[^\r\n]+") do Comms.raid(word) end
    elseif (GetNumPartyMembers()>0) then
      for word in string.gfind(msg, "[^\r\n]+") do Comms.party(word) end
    else
      Comms.console(msg)
    end  
  end,
  
  -- notify msg through console/party/raid
  notify = function(msg)
    Comms.display(msg)
  end,

  notifyDeath = function(msg)
    if (not Status.silentFlag) then
      Comms.display(msg)
    end
  end,

  -- display debug msg
  debug = function (msg)
    if (Status.debugFlag~=4) then -- 4 = off
      if (msg) then
        Comms.console("[DEBUG] "..msg)
      else 
        Comms.console("[DEBUG] nil")
      end
    end
  end,

  -- display msg through console
  console = function (msg)
    if (DEFAULT_CHAT_FRAME) then
      DEFAULT_CHAT_FRAME:AddMessage(msg, 0.0, 1.0, 0.0, 1.0)
    end 
  end,
  
  -- send msg through raid channel
  raid = function(msg)
    SendChatMessage(msg,"RAID") 
  end,

  -- send msg through party channel
  party = function(msg)
    SendChatMessage(msg,"PARTY") 
  end

}


-- =================================================================================================

-- initialize addon
function AnathemaNef_OnLoad()
  SLASH_ATMNEF1 = "/atmnef";
  SlashCmdList["ATMNEF"] = function(msg)
    AnathemaNef_SlashCommandHandler(msg);
  end
	AnathemaNef:RegisterEvent("CHAT_MSG_RAID");
	AnathemaNef:RegisterEvent("CHAT_MSG_PARTY");
  Comms.console("AnathemaNef v" .. AnathemaNef_Version .. " loaded, type /atmnef");
end


-- command handler
function AnathemaNef_SlashCommandHandler(msg)

	-- It's no longer needed to join the channel as we're using the new addon comm system
  -- Comms.join()
  
  msg = string.lower(msg)
  
  -- get channel from possible broadcast command "/atmnef broadcast channel"
  local _, _, broadcast = string.find(msg, "^".."broadcast ".."(.*)")
  
  -- toggle addon mode
  if (msg=="debug") then 
    Status.debug()
  elseif (msg=="silent") then 
    Comms.console("Silently gathering info.")
    Status.silent()
  elseif (msg=="announce") then 
    Status.announce()
  elseif (msg=="report") then 
    Status.report()
  
  -- turn off mod
  elseif (msg=="off") then 
    Status.off()
  
  -- reset counter and start timer  
  elseif (msg=="reset") then   
    Counter.reset()
    Comms.console("Counter reset.")
  
  -- show total
  elseif (msg=="total") then 
    Comms.notify(" Total: "..Counter.getSize())
    Logger.debug("discarded: "..Counter.discarded)
  
  -- show detailed report of all deaths
  elseif (msg=="detail") then
    Comms.notify(Counter.toString())
    --printTable(AnathemaNef_Mobs)

  -- show detailed report of all deaths
  elseif (msg=="record") then
    Counter.record()

  -- check for who's using the addon and what their role is
  elseif (msg=="check") then
    Comms.raid(Comms.CHECK_PREFIX..UnitName("player"))
    Comms.send(Comms.CHECK_PREFIX, UnitName("player"))

  -- show usage
  else Comms.console(Status.usage) end 
  
  -- if it was an actual function of the Status table, display status
  if (Status[string.lower(msg)]) then Comms.console(Status.getStatus()) end
end


-- indirection needed for offline testing
function AnathemaNef_OnEvent()
  AnathemaNef_OnEvent2(event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)
end


-- event handler
function AnathemaNef_OnEvent2(event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)

  -- commented, wait for a command to join the channel
  -- check channel name from variable AnathemaNef_Channel
  --if (event == "VARIABLES_LOADED") then
  --  if (AnathemaNef_Channel) then
  --    Comms.join(AnathemaNef_Channel)
  --  end
  --end

  -- reset counter on nefarius yell
  if ((event == "CHAT_MSG_MONSTER_YELL") and (arg1 == "Let the games begin!")) then
    Counter.reset()
    Comms.console("AnathemaNef: Nefarian phase 1 detected. Counter reset.")
  end 

  -- catch mob death
  if ((event == "CHAT_MSG_COMBAT_HOSTILE_DEATH") and (Status.reportFlag)) then
    local _, _, mobName = string.find(arg1, "(.*) dies.");
    if (mobName) then
    
      -- ignore Bone Construct mobs 
      -- (reporting 42 deaths in 2 secs can get you a kick from the server)
      if (mobName=="Bone Construct") then return end
    
      -- if mobName is indexed, replace mobName with index
      local id = MobData.getIdByName(mobName)
      if (id) then mobName = id end

      Logger.debug("reporting death: msg: "..mobName)
      Comms.send(Comms.ANNOUNCE_DEATH_PREFIX, mobName)
    end
  end
  
  -- Catch death report.
  -- If there is an string on the announce channel, and im on announce mode, 
  -- count this death report if it is not a duplicate  
  if ((event == "CHAT_MSG_ADDON") and (arg1==Comms.ANNOUNCE_DEATH_PREFIX) and (Status.announceFlag)) then
    
    -- get death report
    -- local _, _, mob = string.find(arg1, "^"..Comms.ANNOUNCE_DEATH_PREFIX.."(.*)")
    mob = arg2;
    
    if (mob) then
      Logger.debug("got death report: "..mob)
    
      -- if mob is an Id, replace mob with the real name
      local name = MobData.getNameById(mob)
      if (name) then 
        mob = name 
        Logger.trace("real name for "..mob.." is "..name)
      end

      local added = Counter.add(mob, arg4) -- arg4 is the author of the msg
      if (added) then 
        Comms.notifyDeath(Counter.getSize())
      end
    end    
  end

	-- Catch reply messages
  if ((event == "CHAT_MSG_ADDON") and (arg1==Comms.REPLY_PREFIX) and (Status.announceFlag)) then
    -- process reply msg
    local _, _, reply_name, reply_announce, reply_report = string.find(arg2, "^([a-zA-Z].+) announce=([a-z].+) report=([a-z].+)")
    if (reply_name) then
      Logger.debug("caught a reply msg from "..reply_name.." announce is "..reply_announce.." and report is "..reply_report)
      Comms.display(Comms.CHECK_REPORT_PREFIX..reply_name.." announce="..reply_announce.." report="..reply_report)
    end
	end


	-- Catch status check messages
  if ((event == "CHAT_MSG_ADDON") and (arg1==Comms.CHECK_PREFIX) and ((Status.announceFlag) or (Status.reportFlag)) ) then
      Logger.debug("caught a check msg, replying with our status.")
      local name = UnitName("player")
      local announce = Status.getAsString(Status.announceFlag)
      local report = Status.getAsString(Status.reportFlag)
      Comms.console(Comms.REPLY_PREFIX..name.." announce="..announce.." report="..report)
     	Comms.send(Comms.REPLY_PREFIX, name.." announce="..announce.." report="..report)
	end
    
  
end


