--=============================================================================
-- File:		DKP_System.lua
-- Author:	(synd][cate) 
--          Sammysnake of Ner'Zhul	 2004, 2005 Syndicate Systems
--
-- Description:	Used with eqDKP now includes many DKP related utils. 
-- Now includes Open, Closed, and Roll auction systems as well as bot commands
--
-- Created: April 27, 2005 
-- Modified: April 21, 2006
--=============================================================================

-------------------------------------------
-- Local Variables                       --
-------------------------------------------

local version = "0.27PR8";
--local version = "0.2";

-- Declard color codes for console messages
local RED     = "|cffff0000";
local GREEN   = "|cff00ff00";
local BLUE    = "|cff0000ff";
local MAGENTA = "|cffff00ff";
local YELLOW  = "|cffffff00";
local CYAN    = "|cff00ffff";
local WHITE   = "|cffffffff";


-- GUI Variables

local bBeingDragged;

AuctionEnabled = false;
AuctionList = {};

local RollWarning = true;


----------------------------
-- Global Variables       --
----------------------------

DKP_ROLL_Config = {};
DKP_ROLL_Config.BOT_Options = {};

--=============================================================================
-- Functions


function DKP_Roll_ResetConfig()
		-- Reset DKP_ROLL configuration to default if first time load
		-- Any values that are not in if statements are forced reset next time there is a new verison
		-- Anything inside of ifstatements will cause that value to be loaded if there isn't one already
		
		DKP_ROLL_Config.Version				= version;

		if (DKP_ROLL_Config.Enabled == nil) then		
				DKP_ROLL_Config.Enabled	= true;				
		end
		
		if (DKP_ROLL_Config.Announce == nil) then
				DKP_ROLL_Config.Announce = enabled;
		end
		
		if (DKP_ROLL_Config.Announce_type == nil) then
				DKP_ROLL_Config.Announce_type	= "SAY";
		end
		
		if (DKP_ROLL_Config.PointModifier == nil) then
				DKP_ROLL_Config.PointModifier = 2;
		end
		
		if (DKP_ROLL_Config.System == nil) then
				DKP_ROLL_Config.System				= "ROLL";
		end
		
		if (DKP_ROLL_Config.Channel) then
				DKP_ROLL_Config.Channel				= "NoChannelSet";
		end
				
				
		if (DKP_ROLL_Config.BOT_Enabled == nil) then
				DKP_ROLL_Config.BOT_Enabled = true;
		end
		
		if (DKP_ROLL_Config.BOT_HideWhispers == nil) then
				DKP_ROLL_Config.BOT_HideWhispers = true;		
		end

		if (DKP_ROLL_Config.GUIOpen == nil) then
				DKP_ROLL_Config.GUIOpen = true;		
		end

end


-- Called when DKP_Roll is loaded
function DKP_System_OnLoad() 

		local doLoad = true;   
		local error_noSqlVars = false;
		local error_verMismatch = false;
    
    if (DKP_ROLL_UPDATE_INFO == nil) then
    		doLoad = false;
    		error_noSqlVars = true;

-- Enable once infrared updates process_dkp.php
--    elseif (tonumber(DKP_ROLL_UPDATE_INFO['process_dkp_ver']) ~= tonumber(version)) then
--    		doLoad = false;
--    		error_verMismatch = true;
    else
    		-- No errors;
    end
    
    -- If we didn't encoutner any errors, load stuff and register events !!
    if (doLoad) then
    		
    		-- Savedvariable.lua values load
    		this:RegisterEvent("VARIABLES_LOADED");
    
    		-- allow dragging 
    		--this:RegisterForDrag("RightButtonDown");
    		--this:RegisterForClicks("LeftButtonDown");
    
		    -- respond when roll occurs
		    this:RegisterEvent("CHAT_MSG_SYSTEM");
		    		   
    		-- respond when something is said in chat    
    		this:RegisterEvent("CHAT_MSG_CHANNEL");		    
    		this:RegisterEvent("CHAT_MSG_GUILD");
    		this:RegisterEvent("CHAT_MSG_RAID");
    		this:RegisterEvent("CHAT_MSG_PARTY");
		    
		    
		    -- respond to bot whispers
		    this:RegisterEvent("CHAT_MSG_WHISPER");
		    this:RegisterEvent("CHAT_MSG_WHISPER_INFORM");
		    
		    -- GUI Item Loot
		    this:RegisterEvent("LOOT_OPENED");
		    
		    -- begin with default settings
		    DKP_Roll_ResetConfig();
														
				-- Roll Command Handler
				SLASH_DKPS1 = "/roll";
				SlashCmdList["DKPS"] = function( msg )
						DKP_Roll_SlashCommandHandler( msg );
				end
												
				-- Auction Command Handler
				SLASH_DKPS_AUC1 = "/auc";
				SlashCmdList["DKPS_AUC"] = function( msg )
						DKP_Roll_Auction_SlashCommandHandler( msg );
				end							
		
				-- Display Loaded message
				DEFAULT_CHAT_FRAME:AddMessage(RED .. format(DKPSYS_MESSAGE_DKP_LOADED1,version));
				DEFAULT_CHAT_FRAME:AddMessage(RED .. DKPSYS_MESSAGE_DKP_LOADED2);
				
				-- Display General Usage to Default Chat Frame
				-- Display_General_Usage();
				
				-- Hook Chat Frame messages. (to hide dkp_bot: echo spam)
				Pre_ChatFrame_OnEvent = ChatFrame_OnEvent;
				ChatFrame_OnEvent = DKP_Roll_ChatFrame_OnEvent;				
				
				-- Until we get the time to write the GUI code
				-- Hide the interface
												
		elseif (error_noSqlVars) then
				-- Hide the interface
				DKP_System_GUI_Hide();
		
				-- sqlvars.lua does not exist
				DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_ERROR_SQLVARS_ERROR1);
				DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_ERROR_SQLVARS_ERROR2);
				DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_ERROR_SQLVARS_ERROR3);
				DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_ERROR_SQLVARS_ERROR4);
		elseif (error_verMismatch) then
				DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_ERROR_VER_ERROR1);
				DEFAULT_CHAT_FRAME:AddMessage(format(DKPSYS_ERROR_VER_ERROR2, version));
				DEFAULT_CHAT_FRAME:AddMessage(format(DKPSYS_ERROR_VER_ERROR3, DKP_ROLL_UPDATE_INFO['process_dkp_ver']));
		else
				DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_ERROR_UNKNOWN);
		end
		
end

function DKP_Roll_ChatFrame_OnEvent(event)
		
		local isDKP = false;		

		-- If the DKP_Roll bot is disabled, just return control back to normal hook
		if (not DKP_ROLL_Config.BOT_Enabled) then
				Pre_ChatFrame_OnEvent(event);
				return;
		end
	
		if ( (event == "CHAT_MSG_WHISPER_INFORM") 		-- Is a inform message aka, echo when send someone a tell
				and (DKP_ROLL_Config.BOT_HideWhispers) 		-- Is viewing wispers 
				and (DKP_ROLL_Config.BOT_Enabled) 						-- Bot is enabled
				and (string.find( arg1, whisper_prefix)) ) then		-- Is a DKP_Roll whisper
				--Filter out all dkp whispers to other players
				
				--DEFAULT_CHAT_FRAME:AddMessage(event .. ":".. arg1 .. ":" .. arg2);
				
				isDKP = true;
		end
		
		-- This presumably allows the standard chat to go through,
		-- AFTER the script above.
		if (not isDKP) then
			Pre_ChatFrame_OnEvent(event);
		end		
end

-- Initialize variables if needed
--
-- event	event name
function DKP_System_OnEvent( event )

		local pattern_eng = "(.+) rolls (%d+)";
		local pattern_ger = "(.+) wrfelt. Ergebnis: (%d+)";
		
		local roll_name, roll_num;		
		local use_roll_name, use_roll_num;
		
		local message = arg1;
		local s, e;
		local raid_points, raid_roll;
		local numArgs, result;
		
		local itemName, itemCost;
		local upperName;
		
		local isValidBotUser = false;

		if (event == "VARIABLES_LOADED") then
				if (DKP_ROLL_Config.Version == nil) then
						DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_MESSAGE_DKP_FIRSTRUN);
						DKP_Roll_ResetConfig();
				elseif (DKP_ROLL_Config.Version ~= version) then
						DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_MESSAGE_DKP_PREVINST);
						DKP_Roll_ResetConfig();
				end
				
				if (DKP_ROLL_Config.GUIOpen == true) then
						DKPS_FrameHeader:Show();
				else
						DKPS_FrameHeader:Hide();
				end				
				DKPS_FramePullDown:Hide();
				
				-- Setup GUI Stuff				
				return; -- Don't run code below for normal events.
		end

		if (event == "LOOT_OPENED") then
				ITEMS_DROPDOWN_LIST = {};
				local count = 1;
		
				for i=1, GetNumLootItems(), 1 do						
						ITEMS_DROPDOWN_LIST[count] = {};
						local lootIcon, lootName, lootQuantity, rarity = GetLootSlotInfo(i);						
						
						if (LootSlotIsItem(i)) then																			
								local lootLink = GetLootSlotLink(i);
								if (lootLink ~= nil) then
										--DEFAULT_CHAT_FRAME:AddMessage("ID:" .. i .." LootName:".. lootLink);
										for color, item, name in string.gfind(lootLink, "|c(%x+)|Hitem:(%d+:%d+:%d+:%d+)|h%[(.-)%]|h|r") do
												ITEMS_DROPDOWN_LIST[count].name = "|c".. color .. "[".. name .. "]";
												ITEMS_DROPDOWN_LIST[count].linktext = lootLink;
										end
										count = count + 1;
								end									
						else
								--
						end																														
				end	
				
				if (count == 1) then
						ITEMS_DROPDOWN_LIST[1] = {};
						ITEMS_DROPDOWN_LIST[1].name = "[ No Item ]";
						ITEMS_DROPDOWN_LIST[1].linktext = "[ No Item ]";				
				end

				DKPSystem_GUI_Items_LoadNames();	
									
				return;				
		end
		
		-- Set everything to lowercase so we don't have to worry about that bullshit!
		
		arg1 = string.lower(arg1); -- arguments
		arg2 = string.lower(arg2); -- player name	
		
		if (arg9 ~= nil) then
				arg9 = string.lower(arg9); -- Channel message came from
		end
		
		--if (DKP_ROLL_PLAYERS == nil) then return end

		if  (event and ( (string.find( message, "rolls")) or (string.find( message, "Ergebnis")) )
		and (string.find( message, '%(1%-100%)')) -- A vaild 1-100 roll is detected
		and (DKP_ROLL_Config.Enabled) 						-- Addon is enabled
		and (event == "CHAT_MSG_SYSTEM")) then
		
				if (DKP_ROLL_Config.System == "ROLL") then -- Bidding system is set to ROLL, process roll				
				
						-- Check roll for English Client
						for roll_name, roll_num in string.gfind(message, pattern_eng) do				
								use_roll_name = string.lower(roll_name);				
								use_roll_num = roll_num;
						end
						
						-- Check roll for German Client
						for roll_name, roll_num in string.gfind(message, pattern_ger) do	
								use_roll_name = string.lower(roll_name);				
								use_roll_num = roll_num;
						end
						
						if ( (use_roll_name ~= nil) and (use_roll_num ~= nil) ) then
								-- Display a roll with DKP_Info
								Display_Roll(use_roll_name, use_roll_num);
								
						end
				elseif (RollWarning) then
						DEFAULT_CHAT_FRAME:AddMessage(format(DKPSYS_WARNING_ROLL1, DKP_ROLL_Config.System));
						DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_WARNING_ROLL2);
						DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_WARNING_ROLL3);
						
						-- Only warn the user once about the auction system
						RollWarning = false;
				end
		elseif (((event == "CHAT_MSG_CHANNEL") or ((event == "CHAT_MSG_GUILD") and (DKP_ROLL_Config.Channel == "guild")) or ((event == "CHAT_MSG_RAID") and (DKP_ROLL_Config.Channel == "raid")))
		and (AuctionEnabled) and (DKP_ROLL_Config.System == "OPEN")
		and (tonumber(arg1) ~= nil)) then -- If what was said in chat is just a number (accept this bid)
				-- Auction system is set to OPEN and we are currently open to bidding
				
				if ((arg9 == DKP_ROLL_Config.Channel) or (DKP_ROLL_Config.Channel == "raid") or (DKP_ROLL_Config.Channel == "guild")) then
						-- Something was said in the designated open auction channel
						
						-- DEFAULT_CHAT_FRAME:AddMessage(arg1 ..":".. arg2);
						-- arg1 - what player said
						-- arg2 - playername
						
						---------------------------------------------------
						-- BOT : Auction System (Open) accept bid
						---------------------------------------------------
						
						if (DKP_ROLL_PLAYERS[arg2] == nil) then
								Send_Whisper(arg2, format(DKPSYS_ERROR_PLAYER_DENIED_BID, arg2));
								--DEFAULT_CHAT_FRAME:AddMessage(string.gsub(DKPSYS_ERROR_PLAYER_DENIED_BID, '%%p', arg2));
								return;
						end
						
						local player_bid = tonumber(arg1);
						local player_dkp = tonumber(DKP_ROLL_PLAYERS[arg2]['RaidPoints']);
						
						
						if ((AuctionEnabled) and (DKP_ROLL_Config.System == "OPEN")) then
								if (player_bid > player_dkp) then
										Send_Whisper(arg2, DKPSYS_AUCTION_WISP_INSUFFICENT_DKP1);
										Send_Whisper(arg2, format(DKPSYS_BOT_USERDKP, player_dkp));
								else
										local bidCheckName, bidCheckValue;
										
										-- Check to see if player is upping previous bid
										for bidCheckName, bidCheckValue in AuctionList do
												if (bidCheckName == arg2) then
														Send_Whisper(arg2, format(DKPSYS_AUCTION_WISP_BID_DROPPED, bidCheckValue));
												end
										end
										
										Send_Whisper(arg2, format(DKPSYS_AUCTION_WISP_BID_ACCEPTED, player_bid));
										Send_Whisper(arg2, format(DKPSYS_BOT_USERDKP, player_dkp));
										AuctionList[arg2] = player_bid;
										
										Display_Bidders("GUI");
								end
						elseif ((not AuctionEnabled) and (DKP_ROLL_Config.System == "OPEN")) then
								Send_Whisper(arg2, DKPSYS_AUCTION_WISP_NO_AUCTION);
						else
								Send_Whisper(arg2, DKPSYS_AUCTION_WISP_NOT_OPENMODE);
						end						
						
				end

		elseif ((event == "CHAT_MSG_WHISPER") and (DKP_ROLL_Config.BOT_Enabled)) then
		-----------------------------------------------------------------------------
		-- BOT ARGUMENTS
		-----------------------------------------------------------------------------
				
				-- Split up arguments with " " as the delimiter.
						
				args = GetArgs(arg1, " ");
				
				-- Get the number of arguments
				numArgs = table.getn(args);
				
				if (DKP_ROLL_PLAYERS[arg2] ~= nil) then
						-- Player found in database
						isValidBotUser = true;
				else 
						isValidBotUser = true;
				end
				
				if ((isValidBotUser) and (args[1] == "rp")) then
						-- Set default settings if this is the first time the user has used the bot				
						-- Check to see if ANY user search prefrences exist, if not declare them
						if (DKP_ROLL_Config.BOT_Options == nil) then
								DKP_ROLL_Config.BOT_Options = {};
						end
						
						-- Check to see if specific search preferences exist, if not declare them
						if (DKP_ROLL_Config.BOT_Options[arg2] == nil) then
								DKP_ROLL_Config.BOT_Options[arg2] = {};
						end
						
						local new_user = false;
						-- Check to see if searchmode option is set
						if (DKP_ROLL_Config.BOT_Options[arg2]['searchmode'] == nil) then
								Send_Whisper(arg2, format(DKPSYS_BOT_NEWUSER1, bot_attention));
								DKP_ROLL_Config.BOT_Options[arg2]['searchmode'] = "all";
								new_user = true;
						end
						
						-- Check to see if search threshold is set
					 	if (DKP_ROLL_Config.BOT_Options[arg2]['search_threshold'] == nil) then
					 			Send_Whisper(arg2, format(DKPSYS_BOT_NEWUSER2, bot_attention));
					 			DKP_ROLL_Config.BOT_Options[arg2]['search_threshold'] = 0;
					 			new_user = true;
					 	end						
						
						-- Check to see if search results is set
					 	if (DKP_ROLL_Config.BOT_Options[arg2]['search_results'] == nil) then
					 			Send_Whisper(arg2, format(DKPSYS_BOT_NEWUSER3, bot_attention));
					 			DKP_ROLL_Config.BOT_Options[arg2]['search_results'] = 10;
					 			new_user = true;
					 	end						
		
						-- Since we have a few tew user settings, let the user know they can get help on the command
						if (new_user) then
								Send_Whisper(arg2, format(DKPSYS_BOT_NEWUSER4, bot_attention, bot_attention));
								Send_Whisper(arg2, DKPSYS_BOT_NEWUSER5);
						end
				elseif ((not isValidBotUser) and (args[1] == "rp")) then
						-- Player is not in our database, but wants to use the bot
						Send_Whisper(arg2, DKPSYS_BOT_ACCESSDENIED);
						return;
				else
						-- Player may or may not be valid, but is not using an rp command so ignore it
						
						-- 0.25 bug fix, was causing problems with closed auction since it didn't use RP command
						-- return; 
				end

				-------------------------------------------------------------------------
				-- BOT:SEARCH COMMAND
				-------------------------------------------------------------------------
				-- Checks for arguments of 3 or more, for search queries				
				
				if ((numArgs >= 3) and (args[1] == bot_attention) and (args[2] == "search")) then																
						
						local i;
						local KeyWords = {};
						
						-- Get a table of the keywords we want to look for!
						for i=3, numArgs, 1 do
								--DEFAULT_CHAT_FRAME:AddMessage(i .. ":".. args[1]);
								table.insert(KeyWords, args[i]);
						end
						
						--DEFAULT_CHAT_FRAME:AddMessage(RED .. "DKP_System: " .. WHITE .. "Preforming a search...");
						
						local SearchMode = DKP_ROLL_Config.BOT_Options[arg2]['searchmode'];
						local SearchThreshold = DKP_ROLL_Config.BOT_Options[arg2]['search_threshold'];
						local DisplayResults = DKP_ROLL_Config.BOT_Options[arg2]['search_results'];

						Display_ItemSearchResults(arg2, KeyWords, SearchMode, SearchThreshold, DisplayResults);
						
				elseif (numArgs == 1) then
						---------------------------------------------------
						-- BOT : Commands that use 1 arguments
						---------------------------------------------------												
						if (tonumber(args[1]) ~= nil) then
								
								---------------------------------------------------
								-- BOT : Auction System (Closed) accept bid
								---------------------------------------------------
								local player_bid = tonumber(args[1]);
								local player_dkp = tonumber(DKP_ROLL_PLAYERS[arg2]['RaidPoints']);								
								
								if ((AuctionEnabled) and (DKP_ROLL_Config.System == "CLOSED")) then
										if (player_bid > player_dkp) then
												Send_Whisper(arg2, "You don't have enough DKP to wager that bid!");
												Send_Whisper(arg2, "(Currently you have ".. player_dkp .." DKP)");
										else
												local bidCheckName, bidCheckValue;
												
												-- Check to see if player is upping previous bid
												for bidCheckName, bidCheckValue in AuctionList do
														if (bidCheckName == arg2) then
																Send_Whisper(arg2, format(DKPSYS_AUCTION_WISP_BID_DROPPED, bidCheckValue));
														end
												end
												
												Send_Whisper(arg2, format(DKPSYS_AUCTION_WISP_BID_ACCEPTED, player_bid));
												Send_Whisper(arg2, format(DKPSYS_BOT_USERDKP, player_dkp));
												AuctionList[arg2] = player_bid;
												
												Display_Bidders("GUI");
										end
								elseif ((not AuctionEnabled) and (DKP_ROLL_Config.System == "CLOSED")) then
										Send_Whisper(arg2, DKPSYS_AUCTION_WISP_NO_AUCTION);
								else
										Send_Whisper(arg2, DKPSYS_AUCTION_WISP_NOT_CLOSEDMODE);
								end
								
						elseif (args[1] == bot_attention) then								
								---------------------------------------------------
								-- BOT : Player wants to check their own raidpoints
								---------------------------------------------------
								
								if (DKP_ROLL_PLAYERS[arg2] ~= nil) then
										local pointsSpent = 0;
										local pointsGained = 0;
										
										if (DKP_ROLL_PLAYERS[arg2]['Items'] ~= nil ) then
												for itemName, itemData in DKP_ROLL_PLAYERS[arg2]['Items'] do
														pointsSpent = pointsSpent + itemData['cost'];
												end
										else
												pointsSpent	= 0;
										end
										
										pointsGained = DKP_ROLL_PLAYERS[arg2]['RaidPoints'] + pointsSpent;
								
										Send_Whisper(arg2, format(DKPSYS_BOT_USERDKP_DETAILED, DKP_ROLL_PLAYERS[arg2]['RaidPoints'], pointsSpent, pointsGained));
										Send_Whisper(arg2, format(DKPSYS_BOT_LAST_DB_UPDATE, DKP_ROLL_UPDATE_INFO['date']));
								else
										-- DKP_ROLL_PLAYERS[arg2] is empty, therefore no record exists for that player.
										Send_Whisper(arg2, format(DKPSYS_ERROR_PLAYER_NOT_FOUND,arg2));
								end								
						end -- End of (1) argument commands
				-- Check if command is for DKP, for 2 argument only
				elseif (numArgs == 2) then
						---------------------------------------------------
						-- BOT : Commands that use 2 arguments
						---------------------------------------------------												
					
						if ((string.lower(args[1]) == bot_attention) and (args[2] == "top")) then
								---------------------------------------------------
								-- BOT : (2 args) Display Top 5 user's raidpoints
								---------------------------------------------------
						
								-- Display Top 5 user's raidpoints
								Display_TopRaidPoints(arg2, 10);	
						
						elseif ((args[1] == bot_attention) and ((args[2] == "help") or 
						(args[2] == "?") or	(args[2] == "commands"))) then
								---------------------------------------------------
								-- BOT : (2 args) Display help
								---------------------------------------------------						
						
								Display_BOT_Usage(arg2);						
						
						elseif ((args[1] == bot_attention) and
						(args[2] == "items")) then
								---------------------------------------------------
								-- BOT : (2 args) Display users item History
								---------------------------------------------------						
						
								Display_ItemHistory(arg2);

						elseif ((args[1] == bot_attention) and
						(args[2] == "graids")) then								
								---------------------------------------------------
								-- BOT : (2 args) Display guilds raid History
								---------------------------------------------------							
							
								Display_GuildRaidResults(arg2, nil);
						
						elseif ((args[1] == bot_attention) and
						(args[2] == "raids")) then
								---------------------------------------------------
								-- BOT : (2 args) Displays bot users raid History
								---------------------------------------------------										
								
								Display_GuildRaidResults(arg2, arg2);
						
						
						elseif ((args[1] == bot_attention) and
						(args[2] == "stats")) then
								---------------------------------------------------
								-- BOT : (2 args) Display Database Statistics
								---------------------------------------------------							
						
								Display_Database_Stats(arg2);
						
						elseif (args[1] == bot_attention) then
								---------------------------------------------------
								-- BOT : (2 args) Display rp's for a specified user (arg2)
								---------------------------------------------------							
																
								if (DKP_ROLL_PLAYERS[args[2]] ~= nil) then
								
										local pointsSpent = 0;
										local pointsGained = 0;
										
										if (DKP_ROLL_PLAYERS[args[2]]['Items'] ~= nil ) then
												for itemName, itemData in DKP_ROLL_PLAYERS[args[2]]['Items'] do
														pointsSpent = pointsSpent + itemData['cost'];
												end
										else
												pointsSpent	= 0;
										end
										
										pointsGained = DKP_ROLL_PLAYERS[args[2]]['RaidPoints'] + pointsSpent;								
								
										upperName = string.upper(args[2]);
										
										Send_Whisper(arg2, format(DKPSYS_BOT_USERSDKP_DETAILED, upperName, DKP_ROLL_PLAYERS[args[2]]['RaidPoints'], pointsSpent, pointsGained));
										Send_Whisper(arg2, format(DKPSYS_BOT_LAST_DB_UPDATE, DKP_ROLL_UPDATE_INFO['date']));																			
										
								else								
										Send_Whisper(arg2, format(DKPSYS_ERROR_PLAYER_NOT_FOUND,args[2]));
								end						
						else
								-- ignore, much be chat message.										
						end -- End of (2) argument commands
				-- Check if command is for DKP, for 3 argument only
				elseif (numArgs == 3) then
						---------------------------------------------------
						-- BOT : Commands that use 3 arguments
						---------------------------------------------------								
				
						if ((args[1] == bot_attention) and (args[2] == "top") and 
						element_exists(DKPSYS_Class, args[3])) then
								---------------------------------------------------
								-- BOT : (3 args) Display top raidpoints for <#results> users
								---------------------------------------------------										
								Display_TopRaidPoints(arg2, DKP_ROLL_Config.BOT_Options[arg2]['search_results'], args[3]);
						
						elseif ((args[1] == bot_attention) and (args[2] == "top") 
						and (tonumber(args[3]) == nil)) then
												
								-- If user types "rp top <string>" display error
								-- since string should be a number....
								Send_Whisper(arg2, DKPSYS_ERROR_USAGE);
								Display_BOT_Usage(arg2);						
														
						elseif ((args[1] == bot_attention) and (args[2] == "items")) then
								---------------------------------------------------
								-- BOT : (3 args) Display someone elses items
								---------------------------------------------------										
														
								Display_ItemHistory(arg2, args[3]);								
						
						elseif ((args[1] == bot_attention) and (args[2] == "raids")) then
								---------------------------------------------------
								-- BOT : (2 args) Displays bot users raid History
								---------------------------------------------------										
								
								Display_GuildRaidResults(arg2, args[3]);						
						
						elseif((args[1] == bot_attention) and (args[2] == "raidinfo")) then
						
								if (tonumber(args[3]) ~= nil) then
										Display_RaidStats(arg2, args[3]);
								else
										Send_Whisper(arg2, format(DKPSYS_BOT_ERR_MISMATCH_RAIDINFO, bot_attention));
								end
							
						elseif ( (args[1] == bot_attention) and (args[2] == "searchmode") 
						and ((args[3] == "all") or (args[3] == "any")) ) then
								---------------------------------------------------
								-- BOT : (3 args) SETTINGS: Change search mode
								---------------------------------------------------										
								
								if (args[3] == "all") then
										-- Set matching mode to all words
																																								
										DKP_ROLL_Config.BOT_Options[arg2]['searchmode'] = "all";
										Send_Whisper(arg2, format(DKPSYS_BOT_SEARCH_MODE, "ALL"));
										
										--DEFAULT_CHAT_FRAME:AddMessage(RED .. "DKP_System: ".. WHITE .. "Searching Mode for ".. arg2 ..", set to all words.");
										
								elseif (args[3] == "any") then
										-- Set matching mode to any words										
										
										-- See if Bot Options exist										
										if (DKP_ROLL_Config.BOT_Options == nil) then
												DKP_ROLL_Config.BOT_Options = {};
										end										
										
										-- See if settings for this user exist
										if (DKP_ROLL_Config.BOT_Options[arg2] == nil) then
												DKP_ROLL_Config.BOT_Options[arg2] = {};
										end
										
										DKP_ROLL_Config.BOT_Options[arg2]['searchmode'] = "any";
										Send_Whisper(arg2, format(DKPSYS_BOT_SEARCH_MODE, "ANY"));
										
										--DEFAULT_CHAT_FRAME:AddMessage(RED .. "DKP_Roll: ".. WHITE .. "Searching Mode for ".. arg2 ..", set to any words.");
								else
										-- shouldn't happen
								end
						elseif ( (args[1] == bot_attention) and (args[2] == "searchth") 
						and (tonumber(args[3]) ~= nil)  ) then
								---------------------------------------------------
								-- BOT : (3 args) SETTINGS: Chance search threshold
								---------------------------------------------------								
								-- eg. only search for items over a specific DKP value
																
								DKP_ROLL_Config.BOT_Options[arg2]['search_threshold'] = tonumber(args[3]);
								Send_Whisper(arg2, format(DKPSYS_BOT_SEARCH_THRESHOLD, args[3]));

						elseif ( (args[1] == bot_attention) and (args[2] == "results") 
						and (tonumber(args[3]) ~= nil)  ) then
								---------------------------------------------------
								-- BOT : (3 args) SETTINGS: Chance number of results to return for each query
								---------------------------------------------------			
								
								DKP_ROLL_Config.BOT_Options[arg2]['search_results'] = tonumber(args[3]);
								Send_Whisper(arg2, format(DKPSYS_BOT_SEARCH_RESULTS, args[3]));
								
						elseif ( (args[1] == bot_attention) and (args[2] == "help") ) then

								---------------------------------------------------
								-- BOT : (3 args) HELP: Displays help for a certain command
								---------------------------------------------------			
								
								Send_Whisper(arg2, "Displaying help for the command ".. string.upper(args[3]));
								
								if (args[3] == "top") then
										-- Display help for 'rp help top'
										Send_Whisper(arg2, format(DKPSYS_BOT_HELP_TOP_USAGE_LINE1,bot_attention));
										Send_Whisper(arg2, format(DKPSYS_BOT_HELP_TOP_USAGE_LINE2,bot_attention));
								elseif (args[3] == "items") then
										-- Display help for 'rp help items'
										Send_Whisper(arg2, format(DKPSYS_BOT_HELP_ITEMS_USAGE_LINE1,bot_attention));
										Send_Whisper(arg2, format(DKPSYS_BOT_HELP_ITEMS_USAGE_LINE2,bot_attention));
								elseif (args[3] == "search") then
										-- Display help for 'rp help search'
										Send_Whisper(arg2, format(DKPSYS_BOT_HELP_SEARCH_USAGE_LINE1,bot_attention));
										Send_Whisper(arg2, format(DKPSYS_BOT_HELP_SEARCH_USAGE_LINE2,bot_attention,bot_attention));
										Send_Whisper(arg2, format(DKPSYS_BOT_HELP_SEARCH_USAGE_LINE3,bot_attention));
								elseif (args[3] == "searchmode") then
										-- Display help for 'rp help searchmode'
										Send_Whisper(arg2, format(DKPSYS_BOT_HELP_SEARCHMODE_USAGE_LINE1,bot_attention));
										Send_Whisper(arg2, DKPSYS_BOT_HELP_SEARCHMODE_USAGE_LINE2);
										Send_Whisper(arg2, DKPSYS_BOT_HELP_SEARCHMODE_USAGE_LINE3);
								elseif (args[3] == "searchth") then
										-- Display help for 'rp help searchth'
										Send_Whisper(arg2, format(DKPSYS_BOT_HELP_SEARCHTH_USAGE_LINE1,bot_attention));
										Send_Whisper(arg2, DKPSYS_BOT_HELP_SEARCHTH_USAGE_LINE2);
								elseif (args[3] == "graids") then
										-- Display help for 'rp help graids'
										Send_Whisper(arg2, format(DKPSYS_BOT_HELP_GRAIDS_USAGE_LINE1,bot_attention));
										Send_Whisper(arg2, format(DKPSYS_BOT_HELP_GRAIDS_USAGE_LINE2,bot_attention));
								elseif (args[3] == "raidinfo") then
										-- Display help for 'rp help raidinfo'
										Send_Whisper(arg2, format(DKPSYS_BOT_HELP_RAIDINFO_USAGE_LINE1,bot_attention));
										Send_Whisper(arg2, format(DKPSYS_BOT_HELP_RAIDINFO_USAGE_LINE2,bot_attention));
								elseif (args[3] == "raids") then
										-- Display help for 'rp help raids'
										Send_Whisper(arg2, format(DKPSYS_BOT_HELP_RAIDS_USAGE_LINE1,bot_attention));
										Send_Whisper(arg2, format(DKPSYS_BOT_HELP_RAIDS_USAGE_LINE2,bot_attention));
								else
										-- Invalid command given to find help on
										Send_Whisper(arg2, DKPSYS_ERROR_HELP_USAGE);
										Display_BOT_Usage(arg2);
								end
								
								
						elseif (args[1] == bot_attention) then
								
								-- Unrecognized Command
								Send_Whisper(arg2, DKPSYS_ERROR_USAGE);
								Display_BOT_Usage(arg2);
						else 
								-- Not an rp command
						end
				end												
		end				
		
end

-- ############################################################################################
-- ## Slash Command Handlers

-- Handles all /roll commands
function DKP_Roll_SlashCommandHandler( msg )
	
	local msgArgs;
	local numArgs;
	
	-- Get the arguments
	msg = string.lower(msg);
	msgArgs = GetArgs(msg, " ");
	
	-- Get the number of arguments
	numArgs = table.getn(msgArgs);
				
	-- Check if command is for DKP, for 1 argument only
	if (numArgs == 1) then								
			if (msgArgs[1] == "on") then
				if (DKP_ROLL_Config.Enabled == true) then
						-- Enable Addon
						DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_SLASH_MSG_ALREADY_ENABLED);
						--DKP_ROLL_Config.Announce = false;
				else
						-- Enable Addon
						DKP_ROLL_Config.Enabled = true;
						--DKP_ROLL_Config.Announce = false;		
						DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_SLASH_MSG_ENABLED);
				end
			elseif (msgArgs[1] == "off") then
				if (DKP_ROLL_Config.Enabled == false) then				
						DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_SLASH_MSG_DISABLED);
						DKP_ROLL_Config.Announce = false;
				else
						DKP_ROLL_Config.Enabled = false;
						DKP_ROLL_Config.Announce = false;		
						DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_SLASH_MSG_DISABLED);
				end
			elseif (msgArgs[1] == "announce") then
				if (DKP_ROLL_Config.Announce == true) then
						DKP_ROLL_Config.Announce = false;
						DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_SLASH_MSG_ANNOUNCE_DISABLED);
				else
						DKP_ROLL_Config.Enabled = true;		
						DKP_ROLL_Config.Announce = true;		
						DEFAULT_CHAT_FRAME:AddMessage(format(DKPSYS_SLASH_MSG_ANNOUNCE_ENABLED, DKP_ROLL_Config.Announce_type));
				end
			elseif (msgArgs[1] == "group") then
					DKP_ROLL_Config.Announce_type = "PARTY";
					DKP_ROLL_Config.Announce = true;
					DEFAULT_CHAT_FRAME:AddMessage(format(DKPSYS_SLASH_MSG_ANNOUNCE_TYPE, "party"));		
			elseif (msgArgs[1] == "party") then
					DKP_ROLL_Config.Announce_type = "PARTY";
					DKP_ROLL_Config.Announce = true;
					DEFAULT_CHAT_FRAME:AddMessage(format(DKPSYS_SLASH_MSG_ANNOUNCE_TYPE, "party"));	
			elseif (msgArgs[1] == "raid") then
					DKP_ROLL_Config.Announce_type = "RAID";
					DKP_ROLL_Config.Announce = true;
					DEFAULT_CHAT_FRAME:AddMessage(format(DKPSYS_SLASH_MSG_ANNOUNCE_TYPE, "raid"));				
			elseif (msgArgs[1] == "guild") then
					DKP_ROLL_Config.Announce_type = "GUILD";
					DKP_ROLL_Config.Announce = true;
					DEFAULT_CHAT_FRAME:AddMessage(format(DKPSYS_SLASH_MSG_ANNOUNCE_TYPE, "guild"));
			elseif (msgArgs[1] == "say") then
					DKP_ROLL_Config.Announce_type = "SAY";
					DKP_ROLL_Config.Announce = true;
					DEFAULT_CHAT_FRAME:AddMessage(format(DKPSYS_SLASH_MSG_ANNOUNCE_TYPE, "say"));			
			elseif (msgArgs[1] == "set") then
					-- COMMAND: AUCTION SYSTEM - SET AUCTION SYSTEM TO ROLL
					DKP_ROLL_Config.System = "ROLL";
					DEFAULT_CHAT_FRAME:AddMessage(format(DKPSYS_AUCTION_TYPE, "ROLL"));
			elseif (msgArgs[1] == "help") then
					Display_General_Usage();
			else		
					-- Invalid Command Entered
					DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_SLASH_MSG_INVALID_ARGUMENTS);
					Display_General_Usage();					
			end
	elseif (numArgs == 2) then
			if (msgArgs[1] == "bot") then
					if (msgArgs[2] == "on") then					
							DEFAULT_CHAT_FRAME:AddMessage(format(DKPSYS_BOT_ENABLED, bot_attention));
							DKP_ROLL_Config.BOT_Enabled = true;
					elseif (msgArgs[2] == "off") then							
							DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_BOT_DISABLED);
							DKP_ROLL_Config.BOT_Enabled = false;
					elseif (msgArgs[2] == "hidetell") then												
							DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_BOT_HIDE_WISPER);
							DKP_ROLL_Config.BOT_HideWhispers = true;
					elseif (msgArgs[2] == "showtell") then
							DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_BOT_SHOW_WISPER);
							DKP_ROLL_Config.BOT_HideWhispers = false;
					else
							-- Invalid Command Entered
							Display_General_Usage();
					end
			elseif (msgArgs[1] == "mod") then
					if (tonumber(msgArgs[2]) ~= nil) then
							DKP_ROLL_Config.PointModifier = tonumber(msgArgs[2]);
							DEFAULT_CHAT_FRAME:AddMessage(format(DKPSYS_AUCTION_POINT_MOD, msgArgs[2]));
					else
							-- Invalid Command Entered							
							Display_General_Usage();
							DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_SLASH_MSG_INVALID_ARGUMENTS);
					end
			else
					-- Invalid Command Entered
					DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_SLASH_MSG_INVALID_ARGUMENTS);
					Display_General_Usage();
			end
	elseif (numArgs == 3) then
			if (msgArgs[1] == "sim") then
					if (tonumber(msgArgs[3]) ~= nil) then
							-- Simulate a roll, Display a roll with DKP_Info
							Display_Roll(msgArgs[2], tonumber(msgArgs[3]));										
					else
							-- Invalid Command Entered
							DEFAULT_CHAT_FRAME:AddMessage(RED .."DKP_ROLL: ".. WHITE .."Simulate Roll Usage");							
							DEFAULT_CHAT_FRAME:AddMessage(RED .."DKP_ROLL: ".. WHITE .."/roll sim <playername> <roll>");							
					end
			else					
					-- Invalid Command Entered
					DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_SLASH_MSG_INVALID_ARGUMENTS);
					Display_General_Usage();
			end
	else			
			-- Invalid number of arguments = 0, display usage
			--DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_SLASH_MSG_INVALID_ARGUMENTS);
			Display_General_Usage();
			
			-- /roll now toggles interface on/off, since args = 0 toggle it.
			--DEFAULT_CHAT_FRAME:AddMessage(MSG_GUI_TOGGLED);
			DKP_System_GUI_ToggleVisibility();
							
	end
	
end

-- Handles all /auction commands
function DKP_Roll_Auction_SlashCommandHandler( msg )
	
	local msgArgs;
	local numArgs;
	
	-- Get the arguments
	msg = string.lower(msg);
	msgArgs = GetArgs(msg, " ");
	
	-- Get the number of arguments
	numArgs = table.getn(msgArgs);
				
	-- No arguments entered, start/stop toggle auction
	if (numArgs == 0) then
		
			if (AuctionEnabled == true) then
					AuctionEnabled = false;
					DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_AUCTION_ENDED);
			else
					AuctionEnabled = true;
					DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_AUCTION_STARTED);
					AuctionList = {};
			end	
	-- Check if command is for DKP, for 1 argument only
	elseif (numArgs == 1) then			
			if (msgArgs[1] == "open") then
					-- COMMAND: AUCTION SYSTEM - SET AUCTION SYSTEM TO OPEN
					DKP_ROLL_Config.System = "OPEN";
					DEFAULT_CHAT_FRAME:AddMessage(format(DKPSYS_AUCTION_TYPE, DKPSYS_AUCTION_TYPES.OPEN));
					DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_AUCTION_USAGE_SETCH);
			elseif (msgArgs[1] == "ch") then
					-- COMMAND: AUCTION SYSTEM(OPEN) - DISPLAY CHANNEL WE ARE LISTENING TO
					DEFAULT_CHAT_FRAME:AddMessage(format(DKPSYS_AUCTION_CHANNEL, DKP_ROLL_Config.Channel));
			elseif (msgArgs[1] == "closed") then
					-- COMMAND CATEGORY: AUCTION SYSTEM - SET AUCTION SYSTEM TO CLOSED
					DKP_ROLL_Config.System = "CLOSED";
					DEFAULT_CHAT_FRAME:AddMessage(format(DKPSYS_AUCTION_TYPE, DKPSYS_AUCTION_TYPES.CLOSED));
			elseif (msgArgs[1] == "bids") then
					DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_AUCTION_DISPLAY_BIDS);
					
					Display_Bidders("1");
					
			else
					-- Unrecognized auction command
			end
	elseif (numArgs == 2) then
			if ((msgArgs[1] == "channel") or (msgArgs[1] == "ch")) then					
					-- COMMAND CATEGORY: AUCTION SYSTEM - OPEN AUCTION CHANNEL SET
			
					DKP_ROLL_Config.Channel = msgArgs[2];
					DEFAULT_CHAT_FRAME:AddMessage(format(DKPSYS_AUCTION_CHANNEL_SET, msgArgs[2]));
			elseif (msgArgs[1] == "add") then
					-- For debugging, add a specified username with a random dkp value
					local randValue = random(1,100);
					DEFAULT_CHAT_FRAME:AddMessage("Adding ".. msgArgs[2] .." with a value of ".. randValue);
					
					AuctionList[msgArgs[2]] = randValue;
					
					Display_Bidders("1");
			end
				
	end
	
end

-- ############################################################################################
-- ## Display Functions

-- /auction bids
function Display_Bidders (displayMode)

		local aucPlace, aucBidder, aucValue;
		
		local lastPlaceBid = 0;
		local isTiedText = "";
		local i;
		local SortedBids = {};
		-- AuctionList
		
		-- Reset List
    if (displayMode == "GUI") then  
		    for i=1, 6, 1 do
						local textField = "DKPSystemAuctionButton"..i.."Text";
						getglobal(textField):SetText("");    
		    end
    end		
		
		for aucBidder, aucValue in AuctionList do										
				table.insert(SortedBids, aucBidder);
		end				
		
		table.sort(SortedBids, function (n1, n2)
        return AuctionList[n1] > AuctionList[n2]    -- compare the grades
    end)		
       
    i = 1;
    for aucPlace, aucBidder in SortedBids do
    		if (lastPlaceBid == AuctionList[aucBidder]) then
    				isTiedText = RED .."[TIED]";
    		else
    				isTiedText = "";
    		end    		    		
    		
    		if (displayMode == "GUI") then    		    		
    		    if (i < 6) then
    						local textField = "DKPSystemAuctionButton"..i.."Text";
								getglobal(textField):SetText("  " .. RED .. string.upper(aucBidder) .. WHITE .." ".. AuctionList[aucBidder] .. " DKP ".. isTiedText);
		    		end
    		elseif (displayMode == "TEXT") then
    				DEFAULT_CHAT_FRAME:AddMessage(RED .."DKP_Roll: ".. WHITE .. aucPlace .."] ".. string.upper(aucBidder) .." with a bid of ".. RED .. AuctionList[aucBidder] .. WHITE .." DKP ".. isTiedText);
    		else 
    				-- No argument supplied, use text as default
    				DEFAULT_CHAT_FRAME:AddMessage(RED .."DKP_Roll: ".. WHITE .. aucPlace .."] ".. string.upper(aucBidder) .." with a bid of ".. RED .. AuctionList[aucBidder] .. WHITE .." DKP ".. isTiedText);    				
    		end
    		
    		lastPlaceBid = AuctionList[aucBidder];
    		i = i + 1;
    end

end

function Display_Database_Stats( playerName )
		
		local stats_last_update = DKP_ROLL_UPDATE_INFO['date'];
		local stats_total_players = DKP_ROLL_UPDATE_INFO['total_players'];
		local stats_total_items = DKP_ROLL_UPDATE_INFO['total_items'];
		local stats_total_points = DKP_ROLL_UPDATE_INFO['total_points'];
		local stats_processdkp_ver = DKP_ROLL_UPDATE_INFO['process_dkp_ver'];

		DEFAULT_CHAT_FRAME:AddMessage(RED .."DKP_Roll: ".. WHITE .."Displaying database statistics to ".. playerName);
		
		Send_Whisper(playerName, "DKP Database Statistics");
		Send_Whisper(playerName, "DKP_Roll Version: ".. version .."  Process_dkp.php Version: ".. stats_processdkp_ver);
		Send_Whisper(playerName, "Database Last Updated @ ".. stats_last_update);
		Send_Whisper(playerName, "Total Players:".. stats_total_players .. "  Total Items:".. stats_total_items .."  Total DKP:".. stats_total_points);
			
end

function Display_Roll ( playerName, playerRoll )
		local is_negative = "";
		local raid_roll;
		local playerNameUpper;
		
		local aucPlayer, aucBid;
		local aucFound = false;
		
		
		if (DKP_ROLL_PLAYERS[playerName] ~= nil) then																
				if (DKP_ROLL_PLAYERS[playerName]['RaidPoints'] < 0) then
						is_negative = " (Negative Roll) ";
						raid_roll = (DKP_ROLL_PLAYERS[playerName]['RaidPoints']) + playerRoll;
				else
						raid_roll = (DKP_ROLL_PLAYERS[playerName]['RaidPoints']*DKP_ROLL_Config.PointModifier) + playerRoll;
				end								
				
				DEFAULT_CHAT_FRAME:AddMessage(WHITE .. "DKP_Roll: " .. RED .. playerName .. WHITE .." -- Roll=" .. RED .. playerRoll .. WHITE .. " DKP=" .. RED .. DKP_ROLL_PLAYERS[playerName]['RaidPoints'] .. WHITE .. " DKPB=" .. RED .. (DKP_ROLL_PLAYERS[playerName]['RaidPoints']*DKP_ROLL_Config.PointModifier) .."(".. DKP_ROLL_Config.PointModifier ..")" );								
				DEFAULT_CHAT_FRAME:AddMessage(WHITE .. "DKP_Roll: " .. RED .. playerName .. WHITE .." rolls ".. RED .. raid_roll .. WHITE .." with dkp bonus.");																												
				
				-- If Auction is happening
				if (AuctionEnabled) then
						-- Check to see if the player has already rolled for this auction
						for aucPlayer, aucBid in AuctionList do
								if (aucPlayer == playerName) then
										aucFound = true;
								end
						end		
						
						-- If not add player to auction list
						if (not AucFound) then				
								AuctionList[playerName] = raid_roll;
								DEFAULT_CHAT_FRAME:AddMessage(RED .. "DKP_Roll: " .. WHITE .. "Auction- Adding ".. playerName .." with roll of ".. raid_roll .."!");
						else
								DEFAULT_CHAT_FRAME:AddMessage(RED .. "DKP_Roll: " .. WHITE .. "Auction- ".. RED .. "[Warning] ".. WHITE .. playerName .." has already rolled for current auction!");
						end
						
						Display_Bidders("GUI");
				end
				
				if (DKP_ROLL_Config.Announce) then					
						playerNameUpper = string.upper(playerName);
						SendChatMessage( playerNameUpper .." [".. DKP_ROLL_PLAYERS[playerName]['Class'] .."] R+DKPB = " .. is_negative .. raid_roll .. " R=" .. playerRoll .. " DKP=" .. DKP_ROLL_PLAYERS[playerName]['RaidPoints'] .. " DKPB=" .. (DKP_ROLL_PLAYERS[playerName]['RaidPoints']*DKP_ROLL_Config.PointModifier) .."(".. DKP_ROLL_Config.PointModifier ..")", DKP_ROLL_Config.Announce_type );																				
				end																
				
		else
				DEFAULT_CHAT_FRAME:AddMessage(RED .. "DKP_Roll: User [" .. playerName .."] not found in database, therefore this person may not roll.");								
		end	
end 

--[[
--	Display_GuildRaidResults(msgName);
--	
--	description:
--		- Displays lastest raids encountered by the guild
--
--	args: 
--		- arg1	: [string] msgName - Name of the player to message with the results
--
--	returns:
--		- nil
--]]

function Display_GuildRaidResults(msgName, playerName) 

		local n1,n2;
		local SortedRaids = {};
		local RaidTimeStamps = {};
		local CountResults = 0;
		local NumResults = DKP_ROLL_Config.BOT_Options[arg2]['search_results'];
		local attendPlayer, attendPlayerClass;
		local attendCount = 0;
		local totalRaids = 0;
		
		local raid_id, raid_name, raid_worth, raid_date;
		
		if (playerName == nil) then
				Send_Whisper(msgName, "Displaying last ".. NumResults .." guild raids..");
		else
				Send_Whisper(msgName, "Displaying last ".. NumResults .." raids for ".. string.upper(playerName));
		end
		
		for raidTimeStamp, raidAttribute in DKP_ROLL_RAIDS do										
				table.insert(SortedRaids, raidTimeStamp);
		end				

		table.sort(SortedRaids, function(a,b) return a>b end)    
    
    Send_Whisper(msgName, "[ ID ] __Information__");
		for raidRank, raidTimeStamp in SortedRaids do
				raid_id = DKP_ROLL_RAIDS[raidTimeStamp]['raid_id'];				
				raid_name = DKP_ROLL_RAIDS[raidTimeStamp]['raid_name'];				
				raid_worth = DKP_ROLL_RAIDS[raidTimeStamp]['raid_worth'];
				raid_date = DKP_ROLL_RAIDS[raidTimeStamp]['raid_date'];
				
				if ((playerName == nil) and (CountResults < NumResults)) then
						-- Display guild raids
						Send_Whisper(msgName, "[ ".. raid_id .." ] "..raid_name .."  DKP=".. raid_worth .."  ".. raid_date);
						CountResults = CountResults+1;
				else
						-- Display raids for a certain player
						for attendPlayer, attendPlayerClass in DKP_ROLL_RAIDS[raidTimeStamp]['raid_attendees'] do
								if (attendPlayer == playerName) then
										if (CountResults < NumResults) then
												Send_Whisper(msgName, "[ ".. raid_id .." ] "..raid_name .."  DKP=".. raid_worth .."  ".. raid_date);
												CountResults = CountResults+1;
										end										
										attendCount = attendCount+1;
								end
						end
				end		
				
				totalRaids = totalRaids+1;
		end
		
		if (CountResults == 0) then
				Send_Whisper(msgName, "No attended events for ".. string.upper(playerName) .." found in last ".. totalRaids .." raids!");
		else
				if (playerName ~= nil) then						
						Send_Whisper(msgName, string.upper(playerName) .." attended the last ".. attendCount .." of ".. totalRaids .." raid events.");
				else
						Send_Whisper(msgName, "Finished displaying ".. CountResults .." results of ".. totalRaids .." raid events.");
				end
		end
		
		Send_Whisper(msgName, "Type '".. bot_attention .." raidinfo <id>' to get more info on a specific raid.");
		Send_Whisper(msgName, "Type '".. bot_attention .." results <number>' to change the number of results returned.");
		
end

--[[
--	Display_RaidStats(msgName);
--	
--	description:
--		- Displays lastest raids encountered by the guild
--
--	args: 
--		- arg1	: [string] msgName - Name of the player to message with the results
--
--	returns:
--		- nil
--]]

function Display_RaidStats(msgName, raidID)

		local useTimeStamp = 0;
		local totalAttendance = 0;
		
		local RaidTimeStamp, RaidAttribs;
		local AttribName, AttribValue;
		local className, classValues;
		
		local percent, count;

		for RaidTimeStamp, RaidAttribs in DKP_ROLL_RAIDS do
				
				for AttribName, AttribValue in RaidAttribs do
						if ((AttribName == "raid_id") and (tonumber(AttribValue) == tonumber(raidID)) ) then
								useTimeStamp = RaidTimeStamp;						
						end
				end																
		end
		
		if (useTimeStamp ~= 0) then
				local raid_name = DKP_ROLL_RAIDS[useTimeStamp]['raid_name'];
				local raid_date = DKP_ROLL_RAIDS[useTimeStamp]['raid_date'];
				local raid_worth = DKP_ROLL_RAIDS[useTimeStamp]['raid_worth'];
				--local raid_date = DKP_ROLL_RAIDS[useTimeStamp]['raid_date'];
		
				Send_Whisper(msgName, "Raid Info for [".. raidID .."] ".. raid_name);
				Send_Whisper(msgName, "DATE: ".. raid_date .."  DKPs:".. raid_worth);				
				
				Send_Whisper(msgName, "<< Class Distribution >>");
				for className, classValues in DKP_ROLL_RAIDS[useTimeStamp]['class_distribution'] do
						for classKey, classValue in classValues do
								if (classKey == "percent") then
										percent = classValue;
								elseif (classKey == "total") then								
										count = classValue;
										totalAttendance = totalAttendance + count;
								end																
								--Send_Whisper(msgName, classKey ..":".. classValue);
						end
						Send_Whisper(msgName, "[ ".. count .." ] ".. className .." ".. percent .."%");
				end														
				Send_Whisper(msgName, totalAttendance.. " Players Total.")
				
				-- List Items Dropped
				Send_Whisper(msgName, "<< Items Dropped >>");	
				for ItemName, ItemAttribs in DKP_ROLL_RAIDS[useTimeStamp]['raid_items'] do										
				
						--Send_Whisper(msgName, ItemCost .." RP] ".. Check_Loot_Text(ItemName) .." looted by ".. string.upper(PlayerName));																		
						Send_Whisper(msgName, ItemAttribs['value'] .." RP] ".. Check_Loot_Text(ItemName) .." looted by ".. string.upper(ItemAttribs['value']));																		
						
				end	
		else
				Send_Whisper(msgName, "Error: Raid ID [".. raidID .."] is invalid, type '".. bot_attention .." graids' for reference");				
		end

end

--[[
--	Display_ItemSearchResults(msgName, KeyWords, SearchMode, SearchThreshold, SearchLimit);
--	
--	description:
--		- Displays an Item search given specific keywords to search for, two different modes..
--      If RequireMode is true, all keywords are required for a search to return a result,
--			where as if false returns all items which match any of they keywords.
--
--	args: 
--		- arg1	: [string] msgName - Name of the player to message with the results
--		- arg2	: [array] KeyWords - A table of keywords to search for
--		- arg3  : [string] SearchMode - 'any' or 'all' of the words needs to match
--		- arg4  : [number] SearchThreshold - Searches for items with dkp higher than this number
--
--	returns:
--		- nil
--]]


function Display_ItemSearchResults(msgName, KeyWords, SearchMode, SearchThreshold, SearchLimit)

		local playerName, playerAttrib, playerNameUpper;
		local itemName, itemCost, itemNameUpper;
		
		local AttribName;
		local AttribValue;
		
		local index, currentKeyWord;
		local returnResult, foundResult;
		
		local results = 0;
				
		--DEFAULT_CHAT_FRAME:AddMessage("Keywords:".. KeyWords);
		
		Send_Whisper(msgName, "Searching... (mode='".. SearchMode.."' threshold<=".. SearchThreshold .." results=".. SearchLimit ..")");
		
		for playerName, playerAttrib in DKP_ROLL_PLAYERS do				
			  if (playerAttrib['Items'] ~= nil) then
				for itemName, itemData in playerAttrib['Items'] do										
						
						-- Result Variables
						returnResult = true;
						foundResult = false;
						
						for index, currentKeyWord in KeyWords do		
						
								--DEFAULT_CHAT_FRAME:AddMessage("Current KeyWord:".. currentKeyWord);
								
								if ((string.find( string.lower(itemName), currentKeyWord)) 
								and (itemData['cost'] >= SearchThreshold)) then
											-- Item found matching keyword, send results to user									
											
											playerNameUpper = string.upper(playerName);				
											itemNameUpper = string.upper(itemName);
											
											-- If we are doing an 'any' search we have a result.
											foundResult = true;															
								else
										-- If we are doing an 'all' search, it just failed.
										returnResult = false;
								end								
								
						end
						
						if (returnResult 
						and (SearchMode == "all")) then
								
								-- Only display limited number of results
								if (results < SearchLimit) then
										Send_Whisper(msgName, itemData['cost'] .." RP] ".. Check_Loot_Text(itemName) .." found on ".. playerNameUpper );
								end												
								results = results +1;																																						
						elseif (foundResult 
						and (SearchMode == "any")) then
								-- Only display limited number of results
								if (results < SearchLimit) then
										Send_Whisper(msgName, itemData['cost'] .." RP] ".. Check_Loot_Text(itemName) .." found on ".. playerNameUpper );
								end
								results = results +1;
						else
								-- None of the keywords matched
						end										
				end								
				end
				
				--if (playerAttrib == "Items") then						
				--		for itemName, itemCost in DKP_ROLL_PLAYERS[playerName]['Items'] do						
				--				DEFAULT_CHAT_FRAME:AddMessage(playerName ..":".. itemName);
				--		end						
				--end
		
		end
				
		if (results < SearchLimit) then
				SearchLimit = results;
		end
		
		-- Display how many results were found!!
		Send_Whisper(msgName, "Displaying ".. SearchLimit .." of ".. results .." matching items.");
		Send_Whisper(msgName, "Type '".. bot_attention .." results <number>' to change the number of results displayed!");
		
end

--[[
--	Display_ItemHistory(arg1, arg2, arg3);
--	
--	description:
--		- Displays 
--
--	args: 
--		- arg1	: [string] playername
--		- arg2	: [string] number of results to display
--		- arg3  : [string] what class to display
--
--	returns:
--		- nil
--]]

function Display_ItemHistory(playerName, playerLookup, arg3)
		local playerArray, playerKey, playerValue, playerPoints, playerRank;
		local ItemNames = {};		
		local ItemArray = {};
		local ItemRank, PlayerToLookup;						
		
		local itemName, itemCost;
		local totalRaidPoints = 0;
		
		if (playerLookup ~= nil) then				
				PlayerToLookup = string.lower(playerLookup);
		else				
				PlayerToLookup = string.lower(playerName);
		end
		
		
		if (DKP_ROLL_PLAYERS[PlayerToLookup] == nil) then 
				-- Check to see if player exists in array
				PlayerToLookup = string.upper(PlayerToLookup);
				Send_Whisper(playerName, format(DKPSYS_BOT_ITEM_NOHISTORY, PlayerToLookup));
				return;
		elseif (DKP_ROLL_PLAYERS[PlayerToLookup]['Items'] == nil) then
				-- Check to see if player's items exists in array
				PlayerToLookup = string.upper(PlayerToLookup);
				Send_Whisper(playerName, format(DKPSYS_BOT_ITEM_NOHISTORY, PlayerToLookup));
				return;				
		else				
				-- Do nothin
		end
		
		for itemName, itemData in DKP_ROLL_PLAYERS[PlayerToLookup]['Items'] do
				ItemArray[itemName] = itemData['cost'];
				table.insert(ItemNames, itemName);
		end				
		
		table.sort(ItemNames, function (n1, n2)
        return ItemArray[n1] > ItemArray[n2]    -- compare the grades
    end)
		
		PlayerToLookup = string.upper(PlayerToLookup);
		Send_Whisper(playerName, format(DKPSYS_BOT_ITEM_HISTORY,  PlayerToLookup));
		for itemRank, itemName in ItemNames do		
				if (itemName ~= nil) then
						Send_Whisper(playerName, ItemArray[itemName].." RP] ".. Check_Loot_Text(itemName));
						totalRaidPoints = totalRaidPoints + ItemArray[itemName];
				end
		end
		Send_Whisper(playerName, format(DKPSYS_BOT_ITEM_DKPSPENT, totalRaidPoints));
end


--[[
--	Display_TopRaidPoints(arg1, arg2, arg3);
--	
--	description:
--		- Displays 
--
--	args: 
--		- arg1	: [string] playername
--		- arg2	: [string] number of results to display
--		- arg3  : [string] what class to display
--
--	returns:
--		- nil
--]]

function Display_TopRaidPoints(arg1, NumResults, arg3)
		local playerName, playerArray, playerKey, playerValue, playerPoints, playerRank;
		local PlayerNames = {};		
		local PlayerRaidPoints = {};
		
		local playerNameUpper;
		local a,b,n1,n2;
		
		for playerName, playerArray in DKP_ROLL_PLAYERS do
				
				--DEFAULT_CHAT_FRAME:AddMessage(table.getn(playerArray));
		
				for playerKey, playerValue in playerArray do
						--DEFAULT_CHAT_FRAME:AddMessage(playerName .. ":" .. playerKey .. ":" .. playerValue);
						if (playerKey == "RaidPoints") then
								PlayerRaidPoints[playerName] = playerValue;
								table.insert(PlayerNames, playerName);
						end
				end
		end			
		
		table.sort(PlayerNames, function (n1, n2)
        return PlayerRaidPoints[n1] > PlayerRaidPoints[n2]    -- compare the grades
    end)
            
    if (arg3 ~= nil) then
    		playerNameUpper = string.upper(arg3);  
    		Send_Whisper(arg1, format(DKPSYS_BOT_QUERY_TOPDKP_FOR, NumResults, playerNameUpper));
    else 
    		Send_Whisper(arg1, format(DKPSYS_BOT_QUERY_TOPDKP_OVERALL, NumResults));
    end
    
    a = 1;
		for playerRank, playerName in PlayerNames do
			playerNameUpper = string.upper(playerName);
			
			--DEFAULT_CHAT_FRAME:AddMessage("#"..playerRank .. " " .. playerNameUpper .. " with " .. PlayerRaidPoints[playerName] .. " raidpoints.");
			if (a <= NumResults) then
					if (arg3 ~= nil) then
							-- Top DKP of specific Class
							if (string.lower(arg3) == string.lower(DKP_ROLL_PLAYERS[playerName]['Class'])) then
									Send_Whisper(arg1, format(DKPSYS_BOT_QUERY_DKP, a, playerNameUpper, PlayerRaidPoints[playerName], playerRank));
									a = a + 1;
							elseif (string.lower(arg3) == DKPSYS_Class['cloth']) then
									if ( 	(string.lower(DKP_ROLL_PLAYERS[playerName]['Class']) == DKPSYS_Class.priest) or 
												(string.lower(DKP_ROLL_PLAYERS[playerName]['Class']) == DKPSYS_Class.mage) or
												(string.lower(DKP_ROLL_PLAYERS[playerName]['Class']) == DKPSYS_Class.warlock) ) then
											Send_Whisper(arg1, format(DKPSYS_BOT_QUERY_DKP_CLASS, playerRank, DKP_ROLL_PLAYERS[playerName]['Class'], playerNameUpper, PlayerRaidPoints[playerName]));
											a = a + 1;							
									end
							elseif (string.lower(arg3) == DKPSYS_Class['leather']) then
									if ( 	(string.lower(DKP_ROLL_PLAYERS[playerName]['Class']) == DKPSYS_Class.druid) or 
												(string.lower(DKP_ROLL_PLAYERS[playerName]['Class']) == DKPSYS_Class.roguee) ) then
											Send_Whisper(arg1, format(DKPSYS_BOT_QUERY_DKP_CLASS, playerRank, DKP_ROLL_PLAYERS[playerName]['Class'], playerNameUpper, PlayerRaidPoints[playerName]));
											a = a + 1;							
									end
							elseif (string.lower(arg3) == DKPSYS_Class['mail']) then
									if ( 	(string.lower(DKP_ROLL_PLAYERS[playerName]['Class']) == DKPSYS_Class.shaman) or 
												(string.lower(DKP_ROLL_PLAYERS[playerName]['Class']) == DKPSYS_Class.hunter) ) then
											Send_Whisper(arg1, format(DKPSYS_BOT_QUERY_DKP_CLASS, playerRank, DKP_ROLL_PLAYERS[playerName]['Class'], playerNameUpper, PlayerRaidPoints[playerName]));
											a = a + 1;							
									end		
							elseif (string.lower(arg3) == DKPSYS_Class['plate']) then
									if ( 	(string.lower(DKP_ROLL_PLAYERS[playerName]['Class']) == DKPSYS_Class.warrior) or 
												(string.lower(DKP_ROLL_PLAYERS[playerName]['Class']) == DKPSYS_Class.paladin) ) then
											Send_Whisper(arg1, format(DKPSYS_BOT_QUERY_DKP_CLASS, playerRank, DKP_ROLL_PLAYERS[playerName]['Class'], playerNameUpper, PlayerRaidPoints[playerName]));
											a = a + 1;							
									end																										
							end
					else
							-- Top DKP
							Send_Whisper(arg1, format(DKPSYS_BOT_QUERY_DKP_CLASS, playerRank, DKP_ROLL_PLAYERS[playerName]['Class'], playerNameUpper, PlayerRaidPoints[playerName]));
							a = a + 1;
					end
			end 			
		end

end

--[[
--	Display_General_Usage();
--	
--	description:
--		- Displays DKP_ROLL Usage commands to default chat frame
--
--	args: 
--		- nil
--
--	returns:
--		- nil
--]]

function Display_General_Usage () 
		DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_GENERAL_COMMAND_USAGE_LINE1);
		DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_GENERAL_COMMAND_USAGE_LINE2);
		DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_GENERAL_COMMAND_USAGE_LINE3);
		DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_GENERAL_COMMAND_USAGE_LINE4);
		DEFAULT_CHAT_FRAME:AddMessage(DKPSYS_GENERAL_COMMAND_USAGE_LINE5);
end

--[[
--	Display_BOT_Usage(playerName);
--	
--	description:
--		- Displays DKP_ROLL BOT Usage commands to a specified player name
--
--	args: 
--		- playerName	: [string] Name of the player to message
--
--	returns:
--		- nil
--]]

function Display_BOT_Usage (playerName)
			Send_Whisper(playerName, format(DKPSYS_BOT_COMMAND_USAGE_LINE1, bot_attention));
			Send_Whisper(playerName, DKPSYS_BOT_COMMAND_USAGE_LINE2);
			Send_Whisper(playerName, format(DKPSYS_BOT_COMMAND_USAGE_LINE3, bot_attention, bot_attention));
			Send_Whisper(playerName, format(DKPSYS_BOT_COMMAND_USAGE_LINE4, bot_attention, bot_attention, bot_attention));
			Send_Whisper(playerName, format(DKPSYS_BOT_COMMAND_USAGE_LINE5, bot_attention, bot_attention));
			Send_Whisper(playerName, format(DKPSYS_BOT_COMMAND_USAGE_LINE6, bot_attention));
			Send_Whisper(playerName, format(DKPSYS_BOT_COMMAND_USAGE_LINE7, bot_attention, bot_attention));
			Send_Whisper(playerName, format(DKPSYS_BOT_COMMAND_USAGE_LINE8, bot_attention, bot_attention));
end

--[[
--	Send_Whisper(playerName, Message);
--	
--	description:
--		- Sends an whisper to the designated player.
--
--	args: 
--		- playerName	: [string] Name of the player to message
--		-	Message		: [string] Error that has been preformed
--
--	returns:
--		- True, False
--]]
	
function Send_Whisper(playerName, Message) 		
		
		return SendChatMessage(whisper_prefix .. Message, "WHISPER", this.language, playerName);
end

function Check_Loot_Text (checkItemName) 

		local ItemName, ItemValues;
		local ItemFound = false;
		
		local IMitem;
		local ItemColor, ItemCode;

				
		if ((ItemLinks == nil) and (IMDB == nil) and (ISync == nil)) then
				-- If lootlink isn't installed, just return back with original text
				return checkItemName; 
		end

		if (ISync ~= nil) then
				
				-- "16900:0:0:04:0:14:0:0:2:0:60:0:0",		
				
				if (ISync:FetchLink_ReturnLink(checkItemName) ~= nil) then
						return ISync:FetchLink_ReturnLink(checkItemName);
				end 
				
				local ItemID = ISync:FetchDB(checkItemName,"item");
				if (ItemID ~= nil) then
						local sName, sLink, iQuality, iLevel, sType, sSubType, iCount = GetItemInfo("item:".. ItemID);						
						local QL_Colors = { "ff9d9d9d", "ffffffff", "ff1eff00", "ff0070dd", "ffa335ee", "ffff8000", "ffffcc9d"};	
						
						if (iQuality ~= nil) then
								return "|c".. QL_Colors[iQuality + 1] .."|Hitem:".. ItemID .."|h[".. checkItemName .."]|h|r";
						end			
				else
						return checkItemName;		
				end
				
		-- ItemMatrix Code
		elseif (IMDB ~= nil) then
				--DEFAULT_CHAT_FRAME:AddMessage("ItemMatrix Found");
		
				-- Sort out IM's garble
				-- "ff0070dd14624:0:0:074",
				-- ffffffff6265:0:0:46458008
				local pattern_im_item = "(.+)(.+)";						
				
				for ItemName, ItemValues in IMDB do
						if (string.lower(ItemName) == string.lower(checkItemName)) then														
								--DEFAULT_CHAT_FRAME:AddMessage("Item Found.");						
								
								for ItemColor, ItemCode in string.gfind(IMDB[ItemName], pattern_im_item) do															
										return "|c".. ItemColor .."|Hitem:".. ItemCode .."|h[".. ItemName .."]|h|r";
								end
						end
						
				end																							

		-- LootLink Code
		elseif (ItemLinks ~= nil) then
				for ItemName, ItemValues in ItemLinks do
						if (ItemName == checkItemName) then
								return "|c".. ItemLinks[ItemName]['c'] .."|Hitem:".. ItemLinks[ItemName]['i'] .."|h[".. ItemName .."]|h|r";				
						end
				end
		end
		
		return checkItemName;		
end


-- ############################################################################################
-- ## Miscilanious Functions 

--[[
Extract key/value from message.
]]--
function GetArgs(message, separator)

	-- Declare 'args' variable.
	local args = {};

	-- Declare 'i' integer.
	i = 0;

	-- Search for seperators in the string and return
	-- the separated data.
	for value in string.gfind(message, "[^"..separator.."]+") do
		i = i + 1;
		args[i] = value;
	end -- end for

	-- Submit the filtered data.
	return args;
end -- end GetArgs()


-- ############################################################################################
-- ## Chat Functions 

function LogFloatMsg(msg, color)
    -- Display overhead message.  7 basic colors available
    -- Use at most 3 lines here - the rest get lost
    local r, g, b

    if (color == nil) then 
        color = "white";
    end

    if (string.lower(color) == "red") then
        r, g, b = 1.0, 0.0, 0.0;
    elseif (string.lower(color) == "green") then
        r, g, b = 0.0, 1.0, 0.0;
    elseif (string.lower(color) == "blue") then
        r, g, b = 0.0, 0.0, 1.0;
    elseif (string.lower(color) == "magenta") then
        r, g, b = 1.0, 0.0, 1.0;
    elseif (string.lower(color) == "yellow") then
        r, g, b = 1.0, 1.0, 0.0;
    elseif (string.lower(color) == "cyan") then
        r, g, b = 0.0, 1.0, 1.0;
    else 
        r, g, b = 1.0, 1.0, 1.0;
    end

    UIErrorsFrame:AddMessage(msg, r, g, b, 1.0, UIERRORS_HOLD_TIME);
    
end

-- Check if element exists in an array

function element_exists ( array, element )

	local exists = false;
	for element_name, element_value in array do
			if (element_value == element) then
					exists = true;
			end	
	end
	
	return exists;
end